; ================================================================================================
; Datenausgabe aus RAM an WS2801 bzw. TM1804 (gleichzeitig)
;
; Daten (fr max 65.535 RGB-Pixel) der Reihe nach aus dem RAM ausgeben
; ggfs. fr mehrere Pixel wiederholen (max. 255 mal)
; RGB-Reihenfolge umsortieren nach Bedarf
;
; by Pesi - V 1.0 - 06.04.2011
; ================================================================================================
; benutzte Register      : temp0, temp1, temp2, temp3, temp_R, temp_G, temp_B, YL, YH, XL, XH
;							werden gesichert
; gem. benutzte Register : keine
; reservierte Register   : keine
;
; Konstanten (zum Kopieren):
;
;.equ Pix_Outbuffer		= 0x0060		; RAM-Puffer fr Ausgabedaten
;.equ Pix_Count			= 64			; Pixel-Anzahl
;.equ Pix_Port 			= PortC			; Ausgabeport WS2801/TM1804-Daten
;.equ Pix_DDR 			= DDRC			; DDR WS2801/TM1804-Daten
;.equ Pix_SDO			= 0				; Ausgabepin WS2801/TM1804 Data
;.equ Pix_SCO			= 1				; Ausgabepin WS2801 Clock
; ------------------------------------------------------------------------------------------------

_Data_Shiftout:

		push	temp3							; Register sichern
		push	temp2
		push	temp1
		push	temp0
		push	XL
		push	XH
		push	YL
		push	YH
		push	temp_R
		push	temp_G
		push	temp_B

		ldi		YH, high(PIX_OUTBUFFER)			; Zeiger auf Position Daten im RAM
		ldi		YL, low(PIX_OUTBUFFER)

		lds		XL, RX_FRAMESIZE_L				; Zahl der auszugebenden Bytes in X-Register
		lds		XH, RX_FRAMESIZE_H				; = Zahl der empfangenen Bytes

SUB_DS_Pixcount_Loop:

		ld		temp_R, Y+						; Rot aus RAM laden
		ld		temp_G, Y+						; Grn aus RAM laden
		ld		temp_B, Y+						; Blau aus RAM laden

		ldi		temp1,RGB_ORDER					; Farbreihenfolge holen

		cpi		temp1, 1						; 1 = tauschen RGB -> GRB
		brne	SUB_DS_Swap_1
		mov		temp1, temp_R
		mov		temp_R, temp_G
		mov		temp_G, temp1
		rjmp	SUB_DS_Out

SUB_DS_Swap_1:

		cpi		temp1, 2						; 2 = tauschen RGB -> RBG
		brne	SUB_DS_Swap_2
		mov		temp1, temp_B
		mov		temp_B, temp_G
		mov		temp_G, temp1
		rjmp	SUB_DS_Out

SUB_DS_Swap_2:

		cpi		temp1, 3						; 3 = tauschen RGB -> BGR
		brne	SUB_DS_Swap_3
		mov		temp1, temp_B
		mov		temp_B, temp_R
		mov		temp_R, temp1
		rjmp	SUB_DS_Out

SUB_DS_Swap_3:

		cpi		temp1, 4						; 4 = tauschen RGB -> BRG
		brne	SUB_DS_Swap_4
		mov		temp1, temp_R
		mov		temp_R, temp_B
		mov		temp_B, temp_G
		mov		temp_G, temp1
		rjmp	SUB_DS_Out

SUB_DS_Swap_4:

		cpi		temp1, 5						; 5 = tauschen RGB -> GBR
		brne	SUB_DS_Out
		mov		temp1, temp_R
		mov		temp_R, temp_G
		mov		temp_G, temp_B
		mov		temp_B, temp1
		rjmp	SUB_DS_Out

SUB_DS_Out:

		mov		temp1, temp_R					; Rot ausgeben
		rcall	SUB_DS_Bitout
		mov		temp1, temp_G					; Grn ausgeben
		rcall	SUB_DS_Bitout
		mov		temp1, temp_B					; Blau ausgeben
		rcall	SUB_DS_Bitout

		sbiw	XH:XL, 1						; Pixel zhlen
		breq	SUB_DS_Out_Exit					; alle durch, fertig
		sbiw	XH:XL, 1						; 3x zhlen
		breq	SUB_DS_Out_Exit					; weil 3
		sbiw	XH:XL, 1						; Bytes
		breq	SUB_DS_Out_Exit					; pro Schleifendurchlauf
		rjmp	SUB_DS_Pixcount_Loop			; noch nicht durch, nochmal


SUB_DS_Out_Exit:

		pop		temp_B							; Ja, Register wiederherstellen
		pop		temp_G
		pop		temp_R
		pop		YH
		pop		YL
		pop		XH
		pop		XL
		pop		temp0
		pop		temp1
		pop		temp2
		pop		temp3

		ret										; und zurck

; ================================================================================================
; Ausgabeschleife, gibt den Inhalt von temp1 per Bitbang aus, temp2 wird als Zhler benutzt
; ------------------------------------------------------------------------------------------------

SUB_DS_Bitout:

		ldi		temp2, 8						; Schleife fr 8 Bit Ausgabe


SUB_DS_Bitout_Loop:

		rol		temp1							; auszugebendes Bit in Carry schieben
		brcs	SUB_DS_Out_1					; carry 1? Ja, "1" ausgeben

		cli										; atomar
		sbi		Pix_Port, Pix_SDO				; "0" ausgeben
		nop										; kurzer Puls
		nop
		nop
		nop
		nop
		nop
		cbi		Pix_Port, Pix_SDO				; Puls Ende
		sei										; IR wieder freigeben		
		nop
		nop
		nop
		nop
		sbi		Pix_Port, Pix_SCO				; Clock fr WS2801 pos. Flanke
					
		rjmp	SUB_DS_Out_next_Bit

SUB_DS_Out_1:

		cli										; atomar
		sbi		Pix_Port, Pix_SDO				; "1" ausgeben
		nop										; langer Puls
		nop
		nop
		nop
		nop
		nop
		sbi		Pix_Port, Pix_SCO				; Clock fr WS2801 pos. Flanke
		nop
		nop
		nop
		nop
		nop
		nop
		cbi		Pix_Port, Pix_SDO				; Puls Ende
		sei										; IR wieder freigeben

SUB_DS_Out_next_Bit:

		cbi		Pix_Port, Pix_SCO				; Clock fr WS2801 wieder low

		dec		temp2							; Alle 8 Bit durch...?
		brne	SUB_DS_Bitout_Loop				; Nein, Schleife weiter

		ret										; Ja, zurck

; ================================================================================================
