; ================================================================================================
; tpm2-Empfangsroutine, einfache Version
; ================================================================================================
;
; benutzte Register      : ZL, ZH, werden gesichert, ansonsten itemp-Register
; gem. benutzte Register : rx_timeout
; reservierte Register   : siehe unten
;
; Definitionen (zum Kopieren):
;
; ------------------------------------------------------------------------------------------------

_ISR_Byte_Received_USB:

		in		SREG_Save, SREG					; Statusregister sichern
		push	ZL
		push	ZH

		clr		rx_timeout						; Timeout zurcksetzen
		dec		rx_timeout						; auf 255 (= Zhler starten)

; ------------------------------------------------------------------------------------------------
; Automatische Baudratenerkennung
; ------------------------------------------------------------------------------------------------

		lds		itemp0, UCSR0A					; auf Frame Error prfen
		sbrs	itemp0, FE0
		rjmp	_BR_no_autobaud					; kein FE, weiter

		lds		itemp0, RX_FE_COUNT
		inc		itemp0							; Frame Error zhlen
		sts		RX_FE_COUNT, itemp0				; wieder speichern
		cpi		itemp0, 10						; schon 5 erreicht?
		brlo	_BR_no_autobaud					; nein, weiter 

		lds		itemp0, RX_BAUDRATE				; aktuelle Baudrate holen
		inc		itemp0							; weiterschalten
		cpi		itemp0, 3						; schon grte erreicht?
		brlo	_BR_autobaud					; nein, weiter
		clr		itemp0							; ja, auf 0

_BR_autobaud:

		sts		RX_BAUDRATE, itemp0				; Neue Baudrate
		rcall	_set_baudrate					; einstellen

_BR_no_autobaud:

; ------------------------------------------------------------------------------------------------
; Statusabfrage
; ------------------------------------------------------------------------------------------------

		lds		itemp1, UDR0					; Empfangenes Byte in temp1

		cpi		rx_state, 104					; 104 = Daten speichern
		brne	_BR_check_state_2
		rjmp	_BR_Store_Data

_BR_check_state_2:

.include "Check_Bootstring.inc"					; Auf Startcode Bootloader prfen

		cpi		rx_state, 0						; 0 = warten auf Startcode 0xC9 oder Bootstring
		breq	_BR_Blockstart

		cpi		rx_state, 100					; 100 = Header auswerten
		breq	_BR_Header

		cpi		rx_state, 101					; 101 = Framegre HB empfangen
		breq	_BR_FS_HB

		cpi		rx_state, 102					; 102 = Framegre LB empfangen
		breq	_BR_FS_LB

		cpi		rx_state, 103					; 103 = Auf Startadresse prfen
		breq	_BR_check_startadress

		cpi		rx_state, 106					; 106 = Blockende Daten-Frame prfen
		brne	_BR_check_state_end
		rjmp	_BR_Store_Data_BE

_BR_check_state_end:

		rjmp	_BR_Ende						; Status undefiniert oder Bootloader-Check

; ------------------------------------------------------------------------------------------------
; Warten auf Blockstart
; ------------------------------------------------------------------------------------------------

_BR_Blockstart:

		cpi		itemp1, 0xC9					; Blockstart-Byte empfangen...?
		breq	_BR_Blockstart_OK
		rjmp	_BR_Clear						; nein, Fehler, Receiver reset

_BR_Blockstart_OK:

		ldi		rx_state, 100					; ja, nchster Aufruf: auf Header warten
		rjmp	_BR_Ende						; und raus

; ------------------------------------------------------------------------------------------------
; Header auswerten - 0xDA = Daten empfangen, 0xC0 = Befehl empfangen
; ------------------------------------------------------------------------------------------------

_BR_Header:

		cpi		itemp1, 0xDA					; Daten-Header-Byte empfangen...?
		brne	_BR_Header_check_command		; nein, auf Befehl prfen

		cbr		flags, (1<<FL_FRAME_TYPE)		; ja, Flag auf Daten setzen (0)
		ldi		rx_state, 101					; nchster Aufruf: Framegre HB empfangen

		rjmp	_BR_Ende						; und raus

_BR_Header_check_command:

		cpi		itemp1, 0xC0					; Befehl-Header-Byte empfangen...?
		breq	_BR_Header_OK					; ja, weiter

		rjmp	_BR_Clear						; nein, Receiver Reset

_BR_Header_OK:

		sbr		flags, (1<<FL_FRAME_TYPE)		; Flag auf Befehl setzen (1)
		ldi		rx_state, 101					; nchster Aufruf: Framegre HB empfangen

		rjmp	_BR_Ende						; und raus
	
; ------------------------------------------------------------------------------------------------
; Framegre HB empfangen
; ------------------------------------------------------------------------------------------------

_BR_FS_HB:

		sts		RX_FRAMESIZE_H, itemp1			; HB speichern		
		ldi		rx_state, 102					; nchster Aufruf: Low-Byte empfangen
		rjmp	_BR_Ende						; und raus

; ------------------------------------------------------------------------------------------------
; Framegre LB empfangen
; ------------------------------------------------------------------------------------------------

_BR_FS_LB:

		sts		RX_FRAMESIZE_L, itemp1			; LB speichern		

		tst		itemp1							; LB = 0?
		brne	_BR_FS_LB_OK					; nein, Framegre OK

		lds		itemp0, RX_FRAMESIZE_H
		tst		itemp0							; HB checken
		breq	_BR_Frame_Error					; HB = 0, also Framegre = 0, also Fehler

_BR_FS_LB_OK:

		lds		itemp0, RX_FRAMESIZE_H
		ldi		itemp2, high(DATA_FRAMESIZE_MAX+1)
		cpi		itemp1, low(DATA_FRAMESIZE_MAX+1)	; Auf Max. Framegre prfen
		cpc		itemp0, itemp2
		
		brsh	_BR_Frame_Error					; empfangene Framegre grer als Max., Fehler

		ldi		ZL, low(RX_BUFFER)				; Pointer auf Empfangspuffer setzen
		ldi		ZH, high(RX_BUFFER)
		sts		RX_STOREPOINTER_L, ZL			; und sichern
		sts		RX_STOREPOINTER_H, ZH

		clr		itemp1							; Framecounter resetten
		sts		RX_FRAMECOUNT_L, itemp1
		sts		RX_FRAMECOUNT_H, itemp1

		ldi		rx_state, 104					; nchster Aufruf: Daten speichern

		rjmp	_BR_Ende						; und raus

; ------------------------------------------------------------------------------------------------
; Auf Startadresse prfen
; ------------------------------------------------------------------------------------------------

_BR_check_startadress:

; noch nicht implementiert

; ------------------------------------------------------------------------------------------------
; Daten speichern
; ------------------------------------------------------------------------------------------------

_BR_Store_Data:

		lds		ZL, RX_STOREPOINTER_L			; Pointer holen
		lds		ZH, RX_STOREPOINTER_H

		st		Z+, itemp1						; empfangenes Byte speichern

		sts		RX_STOREPOINTER_L, ZL			; Pointer wieder sichern
		sts		RX_STOREPOINTER_H, ZH

; ------------------------------------------------------------------------------------------------
; Framecounter erhhen und vergleichen
; ------------------------------------------------------------------------------------------------

_BR_count_frames:

		lds		ZL, RX_FRAMECOUNT_L				; Framecounter holen
		lds		ZH, RX_FRAMECOUNT_H

		adiw	ZH:ZL, 1						; erhhen

		sts		RX_FRAMECOUNT_L, ZL				; wieder sichern
		sts		RX_FRAMECOUNT_H, ZH

		lds		itemp1, RX_FRAMESIZE_H			; Framegre holen
		cp		ZH, itemp1						; HB schon erreicht?
		brlo	_BR_Ende						; nein, Ende
		
		lds		itemp1, RX_FRAMESIZE_L			; ja, LB checken
		cp		ZL, itemp1
		brlo	_BR_Ende

		ldi		rx_state, 106					; alle Bytes empfangen, nchster Aufruf Blockende

		rjmp	_BR_Ende						; und raus

; ------------------------------------------------------------------------------------------------
; Blockende fr Datenempfang checken
; ------------------------------------------------------------------------------------------------

_BR_Store_Data_BE:

		cpi		itemp1, 0x36					; Blockende-Byte empfangen...?
		brne	_BR_Frame_Error					; nein, Frame-Error

		sbr		flags, (1<<FL_FRAME_RECEIVED)	; ja, Flag fr Frame empfangen setzen
				
		ldi		itemp1, 0xAC					; Besttigung senden
		sts		UDR0, itemp1

		sbi		SEDU_PIN, S_LED					; Status-LED toggeln

		cbr		Flags, (1<<FL_STANDALONE)		; Flag fr Standalone-Modus lschen

		rjmp	_BR_Clear						; und Receiver Reset

; ------------------------------------------------------------------------------------------------

_BR_Frame_Error:

		ldi		itemp1, 0xEF					; Fehlermeldung senden
		sts		UDR0, itemp1
		clr		rx_timeout						; Timeout zurcksetzen

_BR_Clear:

		clr		rx_state						; Receiver zurcksetzen

_BR_Ende:

		pop		ZH								; Register wiederherstellen
		pop		ZL
		out		SREG, SREG_Save					; Statusregister wiederherstellen


		reti									; Ende Routine

; ================================================================================================
